@@ -16,6 +16,13 @@ from asu.common import get_redis, is_modified
 bp = Blueprint("janitor", __name__)
 
 
+def update_set(key: str, *data: list):
+    pipeline = get_redis().pipeline(True)
+    pipeline.delete(key)
+    pipeline.sadd(key, *data)
+    pipeline.execute()
+
+
 def parse_packages_file(url, repo):
     r = get_redis()
     req = requests.get(url)
@@ -96,13 +103,15 @@ def update_branch(branch):
         current_app.logger.warning("No targets found for {branch['name']}")
         return
 
-    r.sadd(f"targets-{branch['name']}", *list(targets))
+    update_set(f"targets:{branch['name']}", *list(targets))
 
     packages_path = branch["path_packages"].format(branch=branch["name"])
     packages_path = branch["path_packages"].format(branch=branch["name"])
     output_path = current_app.config["JSON_PATH"] / packages_path
     output_path.mkdir(exist_ok=True, parents=True)
 
+    architectures = set()
+
     for version in branch["versions"]:
         current_app.logger.info(f"Update {branch['name']}/{version}")
         # TODO: ugly
@@ -119,7 +128,8 @@ def update_branch(branch):
             update_target_packages(branch, version, target)
 
         for target in targets:
-            update_target_profiles(branch, version, target)
+            if target_arch := update_target_profiles(branch, version, target):
+                architectures.add(target_arch)
 
         overview = {
             "branch": branch["name"],
@@ -144,8 +154,8 @@ def update_branch(branch):
             json.dumps(overview, sort_keys=True, separators=(",", ":"))
         )
 
-    for arch in r.smembers(f"architectures-{branch['name']}"):
-        update_arch_packages(branch, arch.decode("utf-8"))
+    for architecture in architectures:
+        update_arch_packages(branch, architecture)
 
 
 def update_target_packages(branch: dict, version: str, target: str):
@@ -171,7 +181,18 @@ def update_target_packages(branch: dict, version: str, target: str):
 
     current_app.logger.debug(f"{version}/{target}: Found {len(packages)}")
 
-    r.sadd(f"packages-{branch['name']}-{version}-{target}", *list(packages.keys()))
+    update_set(f"packages:{branch['name']}:{version}:{target}", *list(packages.keys()))
+
+    virtual_packages = {
+        vpkg.split("=")[0]
+        for pkg in packages.values()
+        if (provides := pkg.get("provides"))
+        for vpkg in provides.split(", ")
+    }
+    r.sadd(
+        f"packages:{branch['name']}:{version}:{target}",
+        *(virtual_packages | packages.keys()),
+    )
 
     output_path = current_app.config["JSON_PATH"] / version_path / "targets" / target
     output_path.mkdir(exist_ok=True, parents=True)
@@ -194,7 +215,6 @@ def update_target_packages(branch: dict, version: str, target: str):
     )
 
     current_app.logger.info(f"{version}: found {len(package_index.keys())} packages")
-    r.sadd(f"packages-{branch['name']}-{version}", *package_index.keys())
 
 
 def update_arch_packages(branch: dict, arch: str):
@@ -241,10 +261,18 @@ def update_arch_packages(branch: dict, arch: str):
     )
 
     current_app.logger.info(f"{arch}: found {len(package_index.keys())} packages")
-    r.sadd(f"packages-{branch['name']}-{arch}", *package_index.keys())
+    update_set(f"packages:{branch['name']}:{arch}", *package_index.keys())
+
+    virtual_packages = {
+        vpkg.split("=")[0]
+        for pkg in packages.values()
+        if (provides := pkg.get("provides"))
+        for vpkg in provides.split(", ")
+    }
+    r.sadd(f"packages:{branch['name']}:{arch}", *(virtual_packages | packages.keys()))
 
 
-def update_target_profiles(branch: dict, version: str, target: str):
+def update_target_profiles(branch: dict, version: str, target: str) -> str:
     """Update available profiles of a specific version
 
     Args:
@@ -255,38 +283,33 @@ def update_target_profiles(branch: dict, version: str, target: str):
     current_app.logger.info(f"{version}/{target}: Update profiles")
     r = get_redis()
     version_path = branch["path"].format(version=version)
-    req = requests.head(
-        current_app.config["UPSTREAM_URL"]
-        + f"/{version_path}/targets/{target}/profiles.json"
-    )
-
-    if req.status_code != 200:
-        current_app.logger.warning(
-            f"{version}/{target}: Could not download profiles.json"
-        )
-        return
 
     profiles_url = (
         current_app.config["UPSTREAM_URL"]
         + f"/{version_path}/targets/{target}/profiles.json"
     )
 
-    if not is_modified(profiles_url):
-        current_app.logger.debug(f"{version}/{target}: Skip profiles update")
-        return
+    req = requests.get(profiles_url)
 
-    metadata = requests.get(profiles_url).json()
+    if req.status_code != 200:
+        current_app.logger.warning("Couldn't download %s", profiles_url)
+        return False
+
+    metadata = req.json()
     profiles = metadata.pop("profiles", {})
 
-    r.sadd(f"architectures-{branch['name']}", metadata["arch_packages"])
-    r.hset(f"architecture-{branch['name']}", target, metadata["arch_packages"])
+    if not is_modified(profiles_url):
+        current_app.logger.debug(f"{version}/{target}: Skip profiles update")
+        return metadata["arch_packages"]
+
+    r.hset(f"architecture:{branch['name']}", target, metadata["arch_packages"])
 
     queue = Queue(connection=r)
     registry = FinishedJobRegistry(queue=queue)
-    version_code = r.get(f"revision-{version}-{target}")
+    version_code = r.get(f"revision:{version}:{target}")
     if version_code:
         version_code = version_code.decode()
-        for request_hash in r.smembers(f"builds-{version_code}-{target}"):
+        for request_hash in r.smembers(f"builds:{version_code}:{target}"):
             current_app.logger.warning(
                 f"{version_code}/{target}: Delete outdated job build"
             )
@@ -297,26 +320,30 @@ def update_target_profiles(branch: dict, version: str, target: str):
 
             except NoSuchJobError:
                 current_app.logger.warning("Job was already deleted")
-        r.delete(f"build-{version_code}-{target}")
+        r.delete(f"builds:{version_code}:{target}")
 
     r.set(
-        f"revision-{version}-{target}",
+        f"revision:{version}:{target}",
         metadata["version_code"],
     )
 
     current_app.logger.info(f"{version}/{target}: Found {len(profiles)} profiles")
 
+    pipeline = r.pipeline(True)
+    pipeline.delete(f"profiles:{branch['name']}:{version}:{target}")
+
     for profile, data in profiles.items():
         for supported in data.get("supported_devices", []):
-            if not r.hexists(f"mapping-{branch['name']}-{version}-{target}", supported):
+            if not r.hexists(f"mapping:{branch['name']}:{version}:{target}", supported):
                 current_app.logger.info(
                     f"{version}/{target}: Add profile mapping {supported} -> {profile}"
                 )
                 r.hset(
-                    f"mapping-{branch['name']}-{version}-{target}", supported, profile
+                    f"mapping:{branch['name']}:{version}:{target}", supported, profile
                 )
 
-        r.sadd(f"profiles-{branch['name']}-{version}-{target}", profile)
+        pipeline.sadd(f"profiles:{branch['name']}:{version}:{target}", profile)
+
         profile_path = (
             current_app.config["JSON_PATH"]
             / version_path
@@ -342,6 +369,10 @@ def update_target_profiles(branch: dict, version: str, target: str):
 
         data["target"] = target
 
+    pipeline.execute()
+
+    return metadata["arch_packages"]
+
 
 @bp.cli.command("update")
 @click.option("-i", "--interval", default=10, type=int)
