@@ -11,12 +11,22 @@ class BuildCollector(object):
             "Total number of built images",
             labels=["branch", "version", "target", "profile"],
         )
-        for build, count in self.connection.hgetall("stats-builds").items():
+        for build, count in self.connection.hgetall("stats:builds").items():
             stats_builds.add_metric(build.decode().split("#"), count)
 
         yield stats_builds
 
-        hits = self.connection.get("stats-cache-hit")
+        stats_clients = CounterMetricFamily(
+            "clients",
+            "Clients requesting images",
+            labels=["name", "version"],
+        )
+        for client, count in self.connection.hgetall("stats:clients").items():
+            stats_clients.add_metric(client.decode().split("/"), count)
+
+        yield stats_clients
+
+        hits = self.connection.get("stats:cache-hit")
         if hits:
             hits = int(hits.decode())
         else:
@@ -24,7 +34,7 @@ class BuildCollector(object):
 
         yield GaugeMetricFamily("cache_hits", "Cache hits of build images", value=hits)
 
-        misses = self.connection.get("stats-cache-miss")
+        misses = self.connection.get("stats:cache-miss")
 
         if misses:
             misses = int(misses.decode())
@@ -34,14 +44,3 @@ class BuildCollector(object):
         yield GaugeMetricFamily(
             "cache_misses", "Cache misses of build images in percent", value=misses
         )
-
-        extra_package_installs = GaugeMetricFamily(
-            "extra_package_installs",
-            "Package installation that are not installed by default",
-            labels=["package"],
-        )
-
-        for package, count in self.connection.hgetall("stats-packages").items():
-            extra_package_installs.add_metric([package.decode()], count)
-
-        yield extra_package_installs
